#!/bin/sh
# http://www.fftw.org/
# http://www.fftw.org/mailman/listinfo/fftw-announce
# C subroutine library for computing the Discrete Fourier Transform (DFT)
#
# You can build optimized libraries for your maschine with the included build-fftw_lib.sh
# without interfering with package management.
#
# script by : til <rob DOT til DOT freedman AT googlemail DOT com>
set -x -u -e

# 0=no check
ENABLE_CHECK=0

name=fftw
version=3.2.2
build=64.1

arch=i686
tune=pentium-m

pkgname=$name-$version
package=$pkgname-$arch-$build

source="http://www.fftw.org/$pkgname.tar.gz
http://zenwalk.pinguix.com/user-accounts/tilderb/extra/fftw/build-fftw.sh
http://zenwalk.pinguix.com/user-accounts/tilderb/extra/fftw/build-fftw_libs.sh"

CWD=`pwd`
TMP=$CWD/tmp
SRC=$TMP/$pkgname
PKG=$TMP/$package

[ -d $SRC ] && rm -fr $SRC
[ -d $PKG ] && rm -fr $PKG

mkdir -p $SRC $PKG/{install,usr/{src,doc}/$pkgname}

# common to float/double/longdouble;   --with-gcc-arch=pentium-m
OPT_COMMON="\
  --prefix=/usr \
  --mandir=/usr/man \
  --infodir=/usr/info \
  --enable-shared \
  --enable-portable-binary "

# with sse enabled
#OPT_SINGLE='--enable-sse --enable-float'
#OPT_DOUBLE=''
#OPT_LONG_DOUBLE='--enable-long-double'

# with sse2 enabled
OPT_SINGLE='--enable-sse --enable-float'
OPT_DOUBLE='--enable-sse2'
OPT_LONG_DOUBLE='--enable-long-double'

build_fftwlib() {
	mkdir -p tmp_build;
	(	cd tmp_build
		CFLAGS="-O2 -march=$arch -mtune=$tune" \
		../configure $OPT_COMMON ${1}
		make || kill $$
		if [[ $ENABLE_CHECK = 1 ]]; then
		( cd tests; make check || kill $$ )
		fi
		make install DESTDIR=$PKG
	)
	rm -fr tmp_build
}
#
tar xzf $CWD/$pkgname.tar.gz --strip-components=1 -C $SRC
cd $SRC
# build single precision
build_fftwlib "$OPT_SINGLE"

# build double precision
build_fftwlib "$OPT_DOUBLE"

# build long double precision
build_fftwlib "$OPT_LONG_DOUBLE"

# copy docs
gzip -9c ChangeLog >$PKG/usr/doc/$pkgname/ChangeLog.gz
cp -a AUTHORS CONVENTIONS COPYING COPYRIGHT NEWS README TODO \
   $PKG/usr/doc/$pkgname

#( cd doc
#  cp -r html $PKG/usr/doc/$pkg_name-$pkg_ver/
#  cp -r FAQ/fftw-faq.html $PKG/usr/doc/$pkg_name-$pkg_ver/
#)

# strip symbols
find $PKG | xargs file | grep -e "executable" -e "shared object" | grep ELF | cut -f 1 -d : | xargs -r strip --strip-unneeded
find $PKG | xargs file | grep "current ar archive" | cut -s -f 1 -d : | xargs -r strip --strip-debug
#
find $PKG/usr/man -type f -iname '*.?' -exec gzip -9 {} \;
find $PKG/usr/info -type f -iname '*.info' -exec gzip -9 {} \;

# merge dir with /usr/info/dir
[ -e $PKG/usr/info/dir ] && { \

menu="$(sed '1,/Menu:$/d' $PKG/usr/info/dir)"
cat << EOF >>$PKG/install/doinst.sh
#
# Update the /usr/info/dir info-database
#
if ! grep -iq "fftw3" usr/info/dir ; then
  if [ -x usr/bin/install-info ] ; then
    install-info --info-dir=usr/info usr/info/fftw3.info.gz 2>/dev/null
  else
	# add directly 
	echo "$menu" >> usr/info/dir
  fi
fi
EOF
# done with it
mv $PKG/usr/info/dir /tmp
}

# set permissions
find $PKG -type d -perm 777 -exec chmod 755 {} \;

# $pkg_rls script & $pkg_rls optimized library script
cp $CWD/build-fftw.sh $CWD/build-fftw_libs.sh  $PKG/usr/src/$pkgname/

# slack-desc
#   |-----handy-ruler-------------------------------------------------------|
cat <<EOF > $PKG/install/slack-desc
fftw: FFTW Fastest Fourier Transform in the West
fftw:
fftw: FFTW is a free collection libraries for computing the (DFT)
fftw: Discrete Fourier Transform in one or more dimensions.
fftw: It includes complex, real, symmetric, and parallel transforms,
fftw: and can handle arbitrary array sizes efficiently.
fftw:
fftw: http://www.fftw.org/
fftw:
fftw: You can build optimized libraries for your machine with the included
fftw: build-fftw_lib.sh without interfering with package management.
EOF
#
cd $PKG
fakeroot /sbin/makepkg -l y -c n $CWD/$package.txz

cd $CWD
echo "$source" >$package.src
md5sum $package.txz >$package.md5
:>$package.dep
