#!/bin/sh
# http://www.fftw.org/
# http://www.fftw.org/mailman/listinfo/fftw-announce
# C subroutine library for computing the Discrete Fourier Transform (DFT)
#
# You can build optimized libraries for your maschine by running this script.
# without interfering with package management.
# Beware, it takes some time... When any error occured during build,
# no libraries are replaced
#
# script by : til <rob DOT til DOT freedman AT googlemail DOT com>
set -e -x -u

# 0=no check
ENABLE_CHECK=0

CWD=`pwd`

name='fftw'
version='3.2.2'
pkgname=$name-$version
#
if [ "$(id -u)" != "0" ]; then
  echo "This script must be run as root!"; exit 1
fi
#
if [ ! -e $pkgname.tar.gz ]; then
	wget -N http://www.fftw.org/fftw-$version.tar.gz || exit 1
fi
#
TMP=$(tempfile -p fftw- -d ${TMPDIR:-/tmp} )
rm $TMP; mkdir -p $TMP/lib
if [ $? != 0 ]; then
  echo "Error: Unable to create the temporary directory!"; exit 1
fi
trap "echo -e '\nAborting at user request...' ; rm -fr $TMP && exit" 2 15
LIB=$TMP/lib
#
# common to float/double/longdouble
#
OPT_COMMON="--prefix=/usr --enable-shared"
CPU_INFO=$(cat /proc/cpuinfo)

if [ -z $(echo $CPU_INFO | grep -q sse) ]; then
  if [ -z $(echo $CPU_INFO | grep -q sse2) ]; then
    # with sse2 enabled --enable-sse
    OPT_SINGLE='--enable-float'
    OPT_DOUBLE='--enable-sse2'
    OPT_LONG_DOUBLE='--enable-long-double'
  else
    # with sse enabled
    OPT_SINGLE='--enable-sse --enable-float'
    OPT_DOUBLE=''
    OPT_LONG_DOUBLE='--enable-long-double'
  fi
fi

if [ "$(echo "$CPU_INFO" | grep -c "processor")" -gt "1" ]; then
 echo "found multiple processors, enabling threads"
 OPT_COMMON="$OPT_COMMON --enable-threads"
fi

# builds in disposable sub dir
build_fftwlib() {
	mkdir -p tmp_build
	( cd tmp_build
	  ../configure $OPT_COMMON ${1}
	  make || kill $$
	  if [[ $ENABLE_CHECK = 1 ]]; then
	  ( cd tests; make check || kill $$ )
      fi
	  make install-libLTLIBRARIES DESTDIR=$LIB
	)
	rm -fr tmp_build
}
#
log=$CWD/fftwlibs.log

cat <<EOF >$log
$CPU_INFO

C : $OPT_COMMON
S : $OPT_SINGLE
D : $OPT_DOUBLE
L : $OPT_LONG_DOUBLE

EOF

# extract source, building starts
cd $TMP
tar xzf $CWD/$pkgname.tar.gz
cd $pkgname && \
(  # build single precision
  build_fftwlib "$OPT_SINGLE"

  # build double precision
  build_fftwlib "$OPT_DOUBLE"

  # build long double precision
  build_fftwlib "$OPT_LONG_DOUBLE"

  # log sub shell output
) >$log 2>&1

if [[ $(find $LIB -type f -o -type l | wc -l) = 15 ]]; then
# strip symbols
  find $LIB | xargs file | grep "shared object" | grep ELF | cut -s -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
  find $LIB | xargs file | grep "current ar archive" | cut -s -f 1 -d : | xargs strip --strip-debug 2> /dev/null
  cp -uf $LIB/usr/lib/* /usr/lib
  ldconfig
  rm -fr $TMP
else
  echo "Error: check $log for errors"
  echo "$TMP not removed!"
fi
